package de.srsoftware.artdmx;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Random;

public class Light implements Runnable{
    private static final Logger LOG = LoggerFactory.getLogger(Light.class);
    private final byte chn;
    private static final int min = 4;
    private static final int max = 128;
    private static final Random RAND = new Random();
    private final Universe uni;

    public Light(Universe universe, byte channel) {
        LOG.info("creating Light {}",channel);
        chn = channel;
        uni = universe;
    }

    @Override
    public void run() {
        LOG.info("started light {}",chn);
        while (true){
            try {
                long time = 1000+RAND.nextInt(30000);
                LOG.info("light {} sleeping for {}s",chn,time/1000d);
                Thread.sleep(time);
            } catch (InterruptedException e) {
                break;
            }
            for (int bright = min; bright < max; bright++) {
                uni.send(chn,(byte)bright);
                sleep();
            }
            for (int bright = max; bright > min; bright--) {
                uni.send(chn,(byte)bright);
                sleep();
            }

        }
    }

    private static void sleep(){
        try {
            Thread.sleep(33);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
